from PySide6.QtWidgets import QApplication
from QtFrameless import FramelessWindow

class MainWindow(FramelessWindow):
    def __init__(self, parent=None):
        super().__init__(parent=parent)
        self.setWindowTitle("HELLO WORLD!")

if "main" in __name__:
    app = QApplication([])
    window = MainWindow()
    window.show()
    app.exec()



import os
from .qt_api import (
    QApplication,
    QMainWindow,
    QVBoxLayout,
    QWidget,
    Qt
)
from .titleBar import TitleBar
from .style import stylesheet


class FramelessWindow(QMainWindow):
    """A frameless MainWindow widget."""

    def __init__(self, parent=None, titleBarClass=None):
        """Construct new frameless Qt widget."""
        super().__init__(parent=parent)
        self.setWindowFlag(Qt.FramelessWindowHint)
        self.setMouseTracking(True)
        self.setStyleSheet(stylesheet)
        self.setObjectName("MainWindow")
        self._central = QWidget(parent=self)
        self._layout = QVBoxLayout(self._central)
        self._layout.setContentsMargins(0, 0, 0, 0)
        if titleBarClass is not None:
            self.titleBar = titleBarClass()
            self.titleBar.mouseMoveEvent = self._titlebar_mouseMoveEvent
            self.titleBar.mouseReleaseEvent = self._titlebar_mouseReleaseEvent
            self.titleBar.mousePressEvent = self._titlebar_mousePressEvent
            self.titleBar.mouseDoubleClickEvent = self._titlebar_mouseDoubleClickEvent
        else:
            self.titleBar = TitleBar(self)
        self._layout.addWidget(self.titleBar)
        self._layout.addStretch(1)
        self._main = QWidget(self)
        self._layout.addWidget(self._main)
        self.statusbar = self.statusBar()
        super().setCentralWidget(self._central)

    def setTitleBar(self, titleBarClass):
        item = self._layout.takeAt(0)
        item.widget().deleteLater()
        self.titleBar = titleBarClass()
        self._layout.insertWidget(0, self.titleBar)

    def setStyleSheet(self, qss):
        if not isinstance(qss, str):
            if hasattr(qss, "read"):
                qss = qss.read()
            elif hasattr(qss, "readText"):
                qss = qss.readText()
        elif os.path.exists(qss):
            qss = open(qss, "tr").read()
        qss = stylesheet + qss
        super().setStyleSheet(qss)

    def setWindowTitle(self, title: str):
        self.titleBar.setWindowTitle(title)

    def setWindowIcon(self, icon: str):
        self.titleBar.setWindowIcon(icon)

    def setCentralWidget(self, widget):
        item = self._layout.takeAt(2)
        item.widget().deleteLater()
        item.widget().destroy()
        self._layout.takeAt(1)
        widget.mouseMoveEvent = self.mouseMoveEvent
        self._layout.addWidget(widget)

    def _titlebar_mouseDoubleClickEvent(self, _):
        if self.window().isMaximized():
            self.window().showNormal()
        else:
            self.window().showMaximized()

    def _titlebar_mousePressEvent(self, event):
        self._pressed = True
        self._cpos = event.position().toPoint()

    def _titlebar_mouseMoveEvent(self, event):
        if not self._pressed:
            return
        pos = event.position().toPoint()
        difx, dify = (pos - self._cpos).toTuple()
        geom = self.window().geometry()
        x, y, w, h = geom.x(), geom.y(), geom.width(), geom.height()
        new_coords = x+difx, y+dify, w, h
        self.window().setGeometry(*new_coords)

    def _titlebar_mouseReleaseEvent(self, event):
        self._pressed = False
        self._cpos = None

    def setMenuBar(self, menubar):
        """Add a menubar to the titleBar."""
        self.titleBar.setMenuBar(menubar)


def execute():
    app = QApplication([])
    window = FramelessWindow()
    window.show()
    app.exec()



from PySide6.QtWidgets import *
from PySide6.QtCore import *
from PySide6.QtGui import *
from QtFrameless.window import FramelessWindow
import pytest

@pytest.fixture(scope="package")
def app():
    app = QApplication([])
    return app


def test_frameless_window(app):
    window = FramelessWindow()
    assert window.titleBar


def test_frameless_window_subclass(app):
    class Win(FramelessWindow):
        def __init__(self, parent=None):
            super().__init__(parent=parent)
            self.textedit = QTextEdit()
            self.setCentralWidget(self.textedit)

    win = Win()
    assert win.textedit



import sys

if "PyQt6" in sys.modules:
    from PyQt6.QtWidgets import *
    from PyQt6.QtGui import *
    from PyQt6.QtCore import *
elif "PyQt5" in sys.modules:
    from PyQt5.QtWidgets import *
    from PyQt5.QtGui import *
    from PyQt5.QtCore import *
else:
    from PySide6.QtWidgets import *
    from PySide6.QtGui import *
    from PySide6.QtCore import *


QMainWindow = QMainWindow
QWidget = QWidget
QPushButton = QPushButton
QLabel = QLabel
QApplication = QApplication
QVBoxLayout = QVBoxLayout
Qt = Qt
QPixmap = QPixmap
QHBoxLayout = QHBoxLayout
QSizePolicy = QSizePolicy
try:
    Signal = Signal
except:
    Signal = pyqtSignal
QObject = QObject
QMouseEvent = QMouseEvent
QEvent = QEvent



import os
from .qt_api import QLabel, QWidget, QSizePolicy, QHBoxLayout, QPixmap, Qt
from .button import TitleBarCloseButton, TitleBarMaxButton, TitleBarMinButton

icon = os.path.join(os.path.dirname(__file__), "home.png")


class TitleBar(QWidget):
    """Custom titlebar for a frameless QMainWindow."""

    def __init__(self, parent=None):
        """Construct for titlebar."""
        super().__init__(parent=parent)
        self.setObjectName("TitleBarWidget")
        self.setAttribute(Qt.WA_StyledBackground, True)
        self.label = QLabel()
        self.label.setText("TitleBar")
        self.setMaximumHeight(40)
        self.closeButton = TitleBarCloseButton(parent=self)
        self.minimizeButton = TitleBarMinButton(parent=self)
        self.maximizeButton = TitleBarMaxButton(parent=self)
        sizePolicy = QSizePolicy()
        sizePolicy.setHorizontalPolicy(QSizePolicy.Policy.Fixed)
        self.closeButton.setSizePolicy(sizePolicy)
        self.layout = QHBoxLayout(self)
        self.layout.setSpacing(0)
        self.layout.setContentsMargins(0, 0, 0, 0)
        if os.path.exists(icon):
            pix = QPixmap(icon)
        else:
            pix = QPixmap(25,25)
        self.icon = QLabel()
        self.icon.setPixmap(pix)
        self.layout.addWidget(self.icon)
        self.layout.addStretch(1)
        self.layout.addWidget(self.label)
        self.layout.addStretch(1)
        self.layout.addWidget(self.minimizeButton)
        self.layout.addWidget(self.maximizeButton)
        self.layout.addWidget(self.closeButton)
        self.setMouseTracking(True)
        self._pressed = False
        self._cpos = None

    def setWindowTitle(self, title: str) -> None:
        """Sets the title bar label."""
        self.label.setText(title)

    def setWindowIcon(self, icon: str) -> None:
        """Sets the window icon."""
        pixmap = QPixmap(icon)
        self.icon.setPixmap(pixmap)

    def mouseDoubleClickEvent(self, _):
        if self.window().isMaximized():
            self.window().showNormal()
        else:
            self.window().showMaximized()

    def mousePressEvent(self, event):
        self._pressed = True
        self._cpos = event.position().toPoint()

    def mouseMoveEvent(self, event):
        if not self._pressed:
            return
        pos = event.position().toPoint()
        difx, dify = (pos - self._cpos).toTuple()
        geom = self.window().geometry()
        x, y, w, h = geom.x(), geom.y(), geom.width(), geom.height()
        new_coords = x+difx, y+dify, w, h
        self.window().setGeometry(*new_coords)

    def mouseReleaseEvent(self, event):
        self._pressed = False
        self._cpos = None

    def setMenuBar(self, menubar):
        """Set the menu bar in the title bar."""
        self.menuBar = menubar
        self.layout.insertWidget(1, menubar)



stylesheet = """
*[close="true"] {
    border: 1px outset #333;
    border-radius: 9px;
    background-color: #b41605;
}
*[min="true"] {
    border: 1px outset #333;
    border-radius: 9px;
    background-color: #c9c405;
}
*[max="true"] {
    border: 1px outset #333;
    border-radius: 9px;
    background-color: green;
}
*[close="true"]:hover {
    border: 1px outset #333;
    border-radius: 9px;
    background-color: #eb0505;
}
*[min="true"]:hover {
    border: 1px outset #333;
    border-radius: 9px;
    background-color: #f8eb52;
}
*[max="true"]:hover {
    border: 1px outset #333;
    border-radius: 9px;
    background-color: #06e901;
}
*[close="true"]:pressed {
    border: 1px inset #333;
    border-radius: 9px;
    background-color: #c10909;
}
*[min="true"]:pressed {
    border: 1px inset #333;
    border-radius: 9px;
    background-color: #f5e623;
}
*[max="true"]:pressed {
    border: 1px inset #333;
    border-radius: 9px;
    background-color: #22fe1d;
}
"""



from PySide6.QtWidgets import QApplication, QTextEdit
from QtFrameless import FramelessWindow

class MainWindow(FramelessWindow):
    def __init__(self, parent=None):
        super().__init__(parent=parent)
        self.setWindowTitle("Text Editor")
        self.textEdit = QTextEdit(parent=self)
        self.setCentralWidget(self.textEdit)
        self.setStyleSheet("QTextEdit {border: 1px solid black;}")

if "main" in __name__:
    app = QApplication([])
    window = MainWindow()
    window.show()
    app.exec()



import itertools
import random
from PySide6.QtWidgets import QApplication, QPlainTextEdit, QWidget, QPushButton, QVBoxLayout
from QtFrameless import FramelessWindow
import time

colors = [
    "#fff", "#0000ff", "#e8ec35", "#7d189e",
    "#400000", "#222222", "#F51",
    "#33AF11", "#AA1111"
]


class MainWindow(FramelessWindow):
    def __init__(self, parent=None):
        super().__init__(parent=parent)
        self.setWindowTitle("Text Editor")
        self.central = QWidget(parent=self)
        self.textEdit = QPlainTextEdit(parent=self)
        self.layout = QVBoxLayout(self.central)
        self.setCentralWidget(self.central)
        self.layout.addWidget(self.textEdit)
        self.button = QPushButton("DISCO", parent=self)
        self.layout.addWidget(self.button)
        self.button.clicked.connect(self.alternate_colors)

    def alternate_colors(self):
        for c1, c2, c3 in itertools.combinations(colors, 3):
            then = time.time()
            lst = [c1, c2, c3]
            random.shuffle(lst)
            self.setStyleSheet(f"""QWidget {{
                                background-color: {lst[0]};
                                border: 1px solid {lst[1]};
                                color: {lst[2]};}}""")
            app.processEvents()
            while time.time() - then < .25: pass

if "main" in __name__:
    app = QApplication([])
    window = MainWindow()
    window.show()
    window.textEdit.setPlainText("Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum.")
    app.exec()



from .qt_api import QPushButton, QApplication


class TitleBarButton(QPushButton):
    """Standard Buttons for closing and minimizing the window."""

    _action = ""

    def __init__(self, parent=None):
        """Construct the standard window control buttons."""
        super().__init__(parent=parent)
        self.setFixedHeight(20)
        self.setFixedWidth(20)
        self.setObjectName(f"TitleBarButton{self._action}")
        self.clicked.connect(self.window_action)

    def window_action(self):
        """Perform action for the specified button."""
        pass


class TitleBarCloseButton(TitleBarButton):
    """Standard Buttons for closing and minimizing the window."""

    _action = "Close"

    def window_action(self):
        """Perform action for the specified button."""
        self.window().destroy(True, True)
        QApplication.instance().exit()


class TitleBarMinButton(TitleBarButton):
    """Standard Buttons for closing and minimizing the window."""

    _action = "Min"

    def window_action(self):
        """Perform action for the specified button."""
        if self.window().isMinimized():
            self.window().showNormal()
        else:
            self.window().showMinimized()


class TitleBarMaxButton(TitleBarButton):
    """Standard Buttons for closing and minimizing the window."""

    _action = "Max"

    def window_action(self):
        if self.window().isMinimized():
            self.window().showNormal()
        else:
            self.window().showMinimized()



from PySide6.QtWidgets import *
from PySide6.QtCore import *
from PySide6.QtGui import *

from QtFrameless import FramelessWindow

class TitleBar(QWidget):
    def __init__(self, parent=None):
        super().__init__(parent=parent)
        self.layout = QHBoxLayout(self)
        self.setup_menubar()
        self.button = QPushButton("CLOSE", self)
        self.button.clicked.connect(app.exit)
        self.layout.addWidget(self.menu_bar)
        self.layout.addWidget(self.button)
        self.setMaximumHeight(50)

    def setup_menubar(self):
        self.menu_bar = QMenuBar()
        self.file_menu = QMenu("File")
        self.options_menu = QMenu("Options")
        self.edit_menu = QMenu("Edit")
        self.menu_bar.addMenu(self.options_menu)
        self.menu_bar.addMenu(self.file_menu)
        self.menu_bar.addMenu(self.edit_menu)
        self.save_action = QAction("Save")
        self.exit_action = QAction("Exit")
        self.about_action = QAction("About")
        self.copy_action = QAction("Copy")
        self.paste_action = QAction("Paste")
        self.cut_action = QAction("Cut")
        self.file_menu.addActions([self.save_action, self.exit_action])
        self.edit_menu.addActions(
            [self.copy_action, self.cut_action, self.paste_action])
        self.options_menu.addAction(self.about_action)

if "main" in __name__:
    app = QApplication([])
    window = FramelessWindow(titleBarClass=TitleBar)
    window.show()
    app.exec()
